/*
98ppspi.c PC-98 PrinterPort SPI I/O module WPS 2022
DPORT 40h bit0(D0)=CS bit1(D1)=MOSI bit2(D2)=CLK
*/
#include "98ppspi.h"

#define DPORT 0x40
#define CPORT 0x42

#define SPI_CS 0x01
#define SPI_DATA 0x02
#define SPI_CLK 0x04

/*PrinterPort DATA*/
unsigned char _ppspi_data;

/*initialize*/
void ppspi_init()
{
	_ppspi_data = 0xFB;
	outportb(DPORT, _ppspi_data);
}

/*dummy clock clks=clocks*/
void ppspi_dmyclk(unsigned int clks)
{
	unsigned int i = 0;
	for (i = 0; i < clks; i++)
	{
		_ppspi_data |= SPI_CLK;
		outportb(DPORT, _ppspi_data);
		_ppspi_data &= (0xFF ^ SPI_CLK);
		outportb(DPORT, _ppspi_data);
	}
}

/*CS=LOW*/
void ppspi_csl()
{
	_ppspi_data &= (0xFF ^ SPI_CS);
	outportb(DPORT, _ppspi_data);
}

/*CS=HIGH*/
void ppspi_csh()
{
	_ppspi_data |= SPI_CS;
	outportb(DPORT, _ppspi_data);
}

/*Read 1byte*/
unsigned char ppspi_readbyte()
{
	unsigned char r = 0;
	unsigned char b = 0;
	int i = 0;
	for (i = 0; i < 8; i++)
	{
		r = r << 1;
		_ppspi_data |= SPI_DATA;
		_ppspi_data |= SPI_CLK;
		outportb(DPORT, _ppspi_data);
		b = inportb(CPORT);
		if ((b & 0x04) == 0x00)
		{
			r |= 1;
		}
		_ppspi_data &= (0xFF ^ SPI_CLK);
		outportb(DPORT, _ppspi_data);
	}

	return r;
}

/*Write 1byte with read*/
unsigned char ppspi_writebyte(unsigned char c)
{
	unsigned char r = 0;
	unsigned char b = 0;
	unsigned char d = 0x80;
	int i = 0;
	for (i = 0; i < 8; i++)
	{
		r = r << 1;
		if ((c & d) == d)
		{
			_ppspi_data |= SPI_DATA;
		}
		else
		{
			_ppspi_data &= (0xFF ^ SPI_DATA);
		}
		_ppspi_data |= SPI_CLK;
		outportb(DPORT, _ppspi_data);
		b = inportb(CPORT);
		if ((b & 0x04) == 0x00)
		{
			r |= 1;
		}
		_ppspi_data &= (0xFF ^ SPI_CLK);
		outportb(DPORT, _ppspi_data);
		d = d >> 1;
	}

	return r;
}

/*Read bytes*/
void ppspi_read(unsigned char *data, unsigned long size)
{

	unsigned char r = 0;
	unsigned char b = 0;
	int i = 0;
	unsigned long j = 0;

	for (j = 0; j < size; j++)
	{
		r = 0;
		for (i = 0; i < 8; i++)
		{
			r = r << 1;
			_ppspi_data |= SPI_DATA;
			_ppspi_data |= SPI_CLK;
			outportb(DPORT, _ppspi_data);
			b = inportb(CPORT);
			if ((b & 0x04) == 0x00)
			{
				r |= 1;
			}
			_ppspi_data &= (0xFF ^ SPI_CLK);
			outportb(DPORT, _ppspi_data);
		}

		data[j] = r;
	}
}

/*Write bytes*/
void ppspi_write(unsigned char *data, unsigned long size)
{
	unsigned long j = 0;
	unsigned char d = 0x80;
	int i = 0;

	for (j = 0; j < size; j++)
	{
		d = 0x80;
		for (i = 0; i < 8; i++)
		{
			if ((data[j] & d) == d)
			{
				_ppspi_data |= SPI_DATA;
			}
			else
			{
				_ppspi_data &= (0xFF ^ SPI_DATA);
			}
			_ppspi_data |= SPI_CLK;
			outportb(DPORT, _ppspi_data);
			_ppspi_data &= (0xFF ^ SPI_CLK);
			outportb(DPORT, _ppspi_data);
			d = d >> 1;
		}
	}
}

/*Write bytes with read*/
void ppspi_writewithread(unsigned char *outdata,unsigned char *indata, unsigned long size)
{
	unsigned long j = 0;
	unsigned char r = 0;
	unsigned char b = 0;
	unsigned char d = 0x80;
	int i = 0;

	for (j = 0; j < size; j++)
	{
		d = 0x80;
		r = 0;
		for (i = 0; i < 8; i++)
		{
			r = r << 1;
			if ((outdata[j] & d) == d)
			{
				_ppspi_data |= SPI_DATA;
			}
			else
			{
				_ppspi_data &= (0xFF ^ SPI_DATA);
			}
			_ppspi_data |= SPI_CLK;
			outportb(DPORT, _ppspi_data);
			b = inportb(CPORT);
			if ((b & 0x04) == 0x00)
			{
				r |= 1;
			}
			_ppspi_data &= (0xFF ^ SPI_CLK);
			outportb(DPORT, _ppspi_data);
			d = d >> 1;
		}

		indata[j] = r;
	}
}